<?php

namespace BBSSOPro\JWT;

use DateTime;
use DomainException;
use InvalidArgumentException;
use UnexpectedValueException;

/**
 * JSON Web Token implementation, based on this spec:
 * https://tools.ietf.org/html/rfc7519
 *
 * PHP version 5
 *
 * @category Authentication
 * @package  Authentication_JWT
 * @author   Neuman Vong <neuman@twilio.com>
 * @author   Anant Narayanan <anant@php.net>
 * @license  http://opensource.org/licenses/BSD-3-Clause 3-clause BSD
 * @link     https://github.com/firebase/php-jwt
 */
class JWT {

	const ASN1_INTEGER    = 0x02;
	const ASN1_SEQUENCE   = 0x10;
	const ASN1_BIT_STRING = 0x03;

	/**
	 * When checking nbf, iat or expiration times,
	 * we want to provide some extra leeway time to
	 * account for clock skew.
	 */
	public static $leeway = 0;

	/**
	 * Allow the current timestamp to be specified.
	 * Useful for fixing a value within unit testing.
	 *
	 * Will default to PHP time() value if null.
	 */
	public static $timestamp = null;

	public static $supported_algs = array(
		'ES256' => array(
			'openssl',
			'SHA256',
		),
		'HS256' => array(
			'hash_hmac',
			'SHA256',
		),
		'HS384' => array(
			'hash_hmac',
			'SHA384',
		),
		'HS512' => array(
			'hash_hmac',
			'SHA512',
		),
		'RS256' => array(
			'openssl',
			'SHA256',
		),
		'RS384' => array(
			'openssl',
			'SHA384',
		),
		'RS512' => array(
			'openssl',
			'SHA512',
		),
	);

	/**
	 * Decodes a JWT string into a PHP object.
	 *
	 * @param string                $jwt                The JWT
	 * @param string|array|resource $key                The key, or map of keys.
	 *                                                  If the algorithm used is asymmetric, this is the public key
	 * @param array                 $allowed_algs       List of supported verification algorithms
	 *                                                  Supported algorithms are 'ES256', 'HS256', 'HS384', 'HS512',
	 *                                                  'RS256', 'RS384', and 'RS512'
	 *
	 * @return object The JWT's payload as a PHP object
	 *
	 * @throws UnexpectedValueException     Provided JWT was invalid
	 * @throws SignatureInvalidException    Provided JWT was invalid because the signature verification failed
	 * @throws BeforeValidException         Provided JWT is trying to be used before it's eligible as defined by 'nbf'
	 * @throws BeforeValidException         Provided JWT is trying to be used before it's been created as defined by
	 *                                      'iat'
	 * @throws ExpiredException             Provided JWT has since expired, as defined by the 'exp' claim
	 *
	 * @uses jsonDecode
	 * @uses urlsafeB64Decode
	 */
	public static function decode( $jwt, $key, array $allowed_algs = array() ) {
		$timestamp = is_null( static::$timestamp ) ? time() : static::$timestamp;

		if ( empty( $key ) ) {
			throw new InvalidArgumentException( 'Key may not be empty' );
		}
		$tks = explode( '.', $jwt );
		if ( count( $tks ) != 3 ) {
			throw new UnexpectedValueException( 'Wrong number of segments' );
		}
		list($headb64, $bodyb64, $cryptob64) = $tks;
		if ( null === ( $header = static::jsonDecode( static::urlsafeB64Decode( $headb64 ) ) ) ) {
			throw new UnexpectedValueException( 'Invalid header encoding' );
		}
		if ( null === $payload = static::jsonDecode( static::urlsafeB64Decode( $bodyb64 ) ) ) {
			throw new UnexpectedValueException( 'Invalid claims encoding' );
		}
		if ( false === ( $sig = static::urlsafeB64Decode( $cryptob64 ) ) ) {
			throw new UnexpectedValueException( 'Invalid signature encoding' );
		}
		if ( empty( $header->alg ) ) {
			throw new UnexpectedValueException( 'Empty algorithm' );
		}
		if ( empty( static::$supported_algs[ $header->alg ] ) ) {
			throw new UnexpectedValueException( 'Algorithm not supported' );
		}
		if ( ! in_array( $header->alg, $allowed_algs ) ) {
			throw new UnexpectedValueException( 'Algorithm not allowed' );
		}
		if ( $header->alg === 'ES256' ) {
			// OpenSSL expects an ASN.1 DER sequence for ES256 signatures
			$sig = self::signatureToDER( $sig );
		}

		if ( is_array( $key ) || $key instanceof \ArrayAccess ) {
			if ( isset( $header->kid ) ) {
				if ( ! isset( $key[ $header->kid ] ) ) {
					throw new UnexpectedValueException( '"kid" invalid, unable to lookup correct key' );
				}
				$key = $key[ $header->kid ];
			} else {
				throw new UnexpectedValueException( '"kid" empty, unable to lookup correct key' );
			}
		}

		// Check the signature
		if ( ! static::verify( "$headb64.$bodyb64", $sig, $key, $header->alg ) ) {
			throw new SignatureInvalidException( 'Signature verification failed' );
		}

		// Check the nbf if it is defined. This is the time that the
		// token can actually be used. If it's not yet that time, abort.
		if ( isset( $payload->nbf ) && $payload->nbf > ( $timestamp + static::$leeway ) ) {
			throw new BeforeValidException( 'Cannot handle token prior to ' . date( DateTime::ISO8601, $payload->nbf ) );
		}

		// Check that this token has been created before 'now'. This prevents
		// using tokens that have been created for later use (and haven't
		// correctly used the nbf claim).
		if ( isset( $payload->iat ) && $payload->iat > ( $timestamp + static::$leeway ) ) {
			throw new BeforeValidException( 'Cannot handle token prior to ' . date( DateTime::ISO8601, $payload->iat ) );
		}

		// Check if this token has expired.
		if ( isset( $payload->exp ) && ( $timestamp - static::$leeway ) >= $payload->exp ) {
			throw new ExpiredException( 'Expired token' );
		}

		return $payload;
	}

	/**
	 * Converts and signs a PHP object or array into a JWT string.
	 *
	 * @param object|array $payload     PHP object or array
	 * @param string       $key         The secret key.
	 *                                  If the algorithm used is asymmetric, this is the private key
	 * @param string       $alg         The signing algorithm.
	 *                                  Supported algorithms are 'ES256', 'HS256', 'HS384', 'HS512', 'RS256', 'RS384',
	 *                                  and 'RS512'
	 * @param mixed        $keyId
	 * @param array        $head        An array with header elements to attach
	 *
	 * @return string A signed JWT
	 *
	 * @uses jsonEncode
	 * @uses urlsafeB64Encode
	 */
	public static function encode( $payload, $key, $alg = 'HS256', $keyId = null, $head = null ) {
		$header = array(
			'typ' => 'JWT',
			'alg' => $alg,
		);
		if ( $keyId !== null ) {
			$header['kid'] = $keyId;
		}
		if ( isset( $head ) && is_array( $head ) ) {
			$header = array_merge( $head, $header );
		}
		$segments      = array();
		$segments[]    = static::urlsafeB64Encode( static::jsonEncode( $header ) );
		$segments[]    = static::urlsafeB64Encode( static::jsonEncode( $payload ) );
		$signing_input = implode( '.', $segments );

		$signature  = static::sign( $signing_input, $key, $alg );
		$segments[] = static::urlsafeB64Encode( $signature );

		return implode( '.', $segments );
	}

	/**
	 * Sign a string with a given key and algorithm.
	 *
	 * @param string          $msg      The message to sign
	 * @param string|resource $key      The secret key
	 * @param string          $alg      The signing algorithm.
	 *                                  Supported algorithms are 'ES256', 'HS256', 'HS384', 'HS512', 'RS256', 'RS384',
	 *                                  and 'RS512'
	 *
	 * @return string An encrypted message
	 *
	 * @throws DomainException Unsupported algorithm was specified
	 */
	public static function sign( $msg, $key, $alg = 'HS256' ) {
		if ( empty( static::$supported_algs[ $alg ] ) ) {
			throw new DomainException( 'Algorithm not supported' );
		}
		list($function, $algorithm) = static::$supported_algs[ $alg ];
		switch ( $function ) {
			case 'hash_hmac':
				return hash_hmac( $algorithm, $msg, $key, true );
			case 'openssl':
				$signature = '';
				$success   = openssl_sign( $msg, $signature, $key, $algorithm );
				if ( ! $success ) {
					throw new DomainException( 'OpenSSL unable to sign data' );
				} else {
					if ( $alg === 'ES256' ) {
						$signature = self::signatureFromDER( $signature, 256 );
					}

					return $signature;
				}
		}
	}

	/**
	 * Verify a signature with the message, key and method. Not all methods
	 * are symmetric, so we must have a separate verify and sign method.
	 *
	 * @param string          $msg       The original message (header and body)
	 * @param string          $signature The original signature
	 * @param string|resource $key       For HS*, a string key works. for RS*, must be a resource of an openssl public
	 *                                   key
	 * @param string          $alg       The algorithm
	 *
	 * @return bool
	 *
	 * @throws DomainException Invalid Algorithm or OpenSSL failure
	 */
	private static function verify( $msg, $signature, $key, $alg ) {
		if ( empty( static::$supported_algs[ $alg ] ) ) {
			throw new DomainException( 'Algorithm not supported' );
		}

		list($function, $algorithm) = static::$supported_algs[ $alg ];
		switch ( $function ) {
			case 'openssl':
				$success = openssl_verify( $msg, $signature, $key, $algorithm );
				if ( $success === 1 ) {
					return true;
				} elseif ( $success === 0 ) {
					return false;
				}
				// returns 1 on success, 0 on failure, -1 on error.
				throw new DomainException( 'OpenSSL error: ' . openssl_error_string() );
			case 'hash_hmac':
			default:
				$hash = hash_hmac( $algorithm, $msg, $key, true );
				if ( function_exists( 'hash_equals' ) ) {
					return hash_equals( $signature, $hash );
				}
				$len = min( static::safeStrlen( $signature ), static::safeStrlen( $hash ) );

				$status = 0;
				for ( $i = 0; $i < $len; $i++ ) {
					$status |= ( ord( $signature[ $i ] ) ^ ord( $hash[ $i ] ) );
				}
				$status |= ( static::safeStrlen( $signature ) ^ static::safeStrlen( $hash ) );

				return ( $status === 0 );
		}
	}

	/**
	 * Decode a JSON string into a PHP object.
	 *
	 * @param string $input JSON string
	 *
	 * @return object Object representation of JSON string
	 *
	 * @throws DomainException Provided string was invalid JSON
	 */
	public static function jsonDecode( $input ) {
		if ( version_compare( PHP_VERSION, '5.4.0', '>=' ) && ! ( defined( 'JSON_C_VERSION' ) && PHP_INT_SIZE > 4 ) ) {
			/** In PHP >=5.4.0, json_decode() accepts an options parameter, that allows you
			 * to specify that large ints (like Steam Transaction IDs) should be treated as
			 * strings, rather than the PHP default behaviour of converting them to floats.
			 */
			$obj = json_decode( $input, false, 512, JSON_BIGINT_AS_STRING );
		} else {
			/** Not all servers will support that, however, so for older versions we must
			 * manually detect large ints in the JSON string and quote them (thus converting
			 *them to strings) before decoding, hence the preg_replace() call.
			 */
			$max_int_length       = strlen( (string) PHP_INT_MAX ) - 1;
			$json_without_bigints = preg_replace( '/:\s*(-?\d{' . $max_int_length . ',})/', ': "$1"', $input );
			$obj                  = json_decode( $json_without_bigints );
		}

		if ( $errno = json_last_error() ) {
			static::handleJsonError( $errno );
		} elseif ( $obj === null && $input !== 'null' ) {
			throw new DomainException( 'Null result with non-null input' );
		}

		return $obj;
	}

	/**
	 * Encode a PHP object into a JSON string.
	 *
	 * @param object|array $input A PHP object or array
	 *
	 * @return string JSON representation of the PHP object or array
	 *
	 * @throws DomainException Provided object could not be encoded to valid JSON
	 */
	public static function jsonEncode( $input ) {
		$json = json_encode( $input );
		if ( $errno = json_last_error() ) {
			static::handleJsonError( $errno );
		} elseif ( $json === 'null' && $input !== null ) {
			throw new DomainException( 'Null result with non-null input' );
		}

		return $json;
	}

	/**
	 * Decode a string with URL-safe Base64.
	 *
	 * @param string $input A Base64 encoded string
	 *
	 * @return string A decoded string
	 */
	public static function urlsafeB64Decode( $input ) {
		$remainder = strlen( $input ) % 4;
		if ( $remainder ) {
			$padlen = 4 - $remainder;
			$input .= str_repeat( '=', $padlen );
		}

		return base64_decode( strtr( $input, '-_', '+/' ) );
	}

	/**
	 * Encode a string with URL-safe Base64.
	 *
	 * @param string $input The string you want encoded
	 *
	 * @return string The base64 encode of what you passed in
	 */
	public static function urlsafeB64Encode( $input ) {
		return str_replace( '=', '', strtr( base64_encode( $input ), '+/', '-_' ) );
	}

	/**
	 * Helper method to create a JSON error.
	 *
	 * @param int $errno An error number from json_last_error()
	 *
	 * @return void
	 */
	private static function handleJsonError( $errno ) {
		$messages = array(
			JSON_ERROR_DEPTH          => 'Maximum stack depth exceeded',
			JSON_ERROR_STATE_MISMATCH => 'Invalid or malformed JSON',
			JSON_ERROR_CTRL_CHAR      => 'Unexpected control character found',
			JSON_ERROR_SYNTAX         => 'Syntax error, malformed JSON',
			JSON_ERROR_UTF8           => 'Malformed UTF-8 characters',
			// PHP >= 5.3.3
		);
		throw new DomainException( isset( $messages[ $errno ] ) ? $messages[ $errno ] : 'Unknown JSON error: ' . $errno );
	}

	/**
	 * Get the number of bytes in cryptographic strings.
	 *
	 * @param string $str
	 *
	 * @return int
	 */
	private static function safeStrlen( $str ) {
		if ( function_exists( 'mb_strlen' ) ) {
			return mb_strlen( $str, '8bit' );
		}

		return strlen( $str );
	}

	/**
	 * Convert an ECDSA signature to an ASN.1 DER sequence
	 *
	 * @param string $sig The ECDSA signature to convert
	 *
	 * @return  string The encoded DER object
	 */
	private static function signatureToDER( $sig ) {
		// Separate the signature into r-value and s-value
		list($r, $s) = str_split( $sig, (int) ( strlen( $sig ) / 2 ) );

		// Trim leading zeros
		$r = ltrim( $r, "\x00" );
		$s = ltrim( $s, "\x00" );

		// Convert r-value and s-value from unsigned big-endian integers to
		// signed two's complement
		if ( ord( $r[0] ) > 0x7f ) {
			$r = "\x00" . $r;
		}
		if ( ord( $s[0] ) > 0x7f ) {
			$s = "\x00" . $s;
		}

		return self::encodeDER( self::ASN1_SEQUENCE, self::encodeDER( self::ASN1_INTEGER, $r ) . self::encodeDER( self::ASN1_INTEGER, $s ) );
	}

	/**
	 * Encodes a value into a DER object.
	 *
	 * @param int    $type  DER tag
	 * @param string $value the value to encode
	 *
	 * @return  string  the encoded object
	 */
	private static function encodeDER( $type, $value ) {
		$tag_header = 0;
		if ( $type === self::ASN1_SEQUENCE ) {
			$tag_header |= 0x20;
		}

		// Type
		$der = chr( $tag_header | $type );

		// Length
		$der .= chr( strlen( $value ) );

		return $der . $value;
	}

	/**
	 * Encodes signature from a DER object.
	 *
	 * @param string $der     binary signature in DER format
	 * @param int    $keySize the nubmer of bits in the key
	 *
	 * @return  string  the signature
	 */
	private static function signatureFromDER( $der, $keySize ) {
		// OpenSSL returns the ECDSA signatures as a binary ASN.1 DER SEQUENCE
		list($offset, $_) = self::readDER( $der );
		list($offset, $r) = self::readDER( $der, $offset );
		list($offset, $s) = self::readDER( $der, $offset );

		// Convert r-value and s-value from signed two's compliment to unsigned
		// big-endian integers
		$r = ltrim( $r, "\x00" );
		$s = ltrim( $s, "\x00" );

		// Pad out r and s so that they are $keySize bits long
		$r = str_pad( $r, $keySize / 8, "\x00", STR_PAD_LEFT );
		$s = str_pad( $s, $keySize / 8, "\x00", STR_PAD_LEFT );

		return $r . $s;
	}

	/**
	 * Reads binary DER-encoded data and decodes into a single object
	 *
	 * @param string $der    the binary data in DER format
	 * @param int    $offset the offset of the data stream containing the object
	 *                       to decode
	 *
	 * @return array [$offset, $data] the new offset and the decoded object
	 */
	private static function readDER( $der, $offset = 0 ) {
		$pos         = $offset;
		$size        = strlen( $der );
		$constructed = ( ord( $der[ $pos ] ) >> 5 ) & 0x01;
		$type        = ord( $der[ $pos++ ] ) & 0x1f;

		// Length
		$len = ord( $der[ $pos++ ] );
		if ( $len & 0x80 ) {
			$n   = $len & 0x1f;
			$len = 0;
			while ( $n-- && $pos < $size ) {
				$len = ( $len << 8 ) | ord( $der[ $pos++ ] );
			}
		}

		// Value
		if ( $type == self::ASN1_BIT_STRING ) {
			$pos++; // Skip the first contents octet (padding indicator)
			$data = substr( $der, $pos, $len - 1 );
			$pos += $len - 1;
		} elseif ( ! $constructed ) {
			$data = substr( $der, $pos, $len );
			$pos += $len;
		} else {
			$data = null;
		}

		return array(
			$pos,
			$data,
		);
	}
}
